﻿<#
.SYNOPSIS
	Automates all steps required to create a new Lync Room System room account. Account can optionally
	be Enterprise Voice enabled.

.DESCRIPTION
	Automates all steps required to create a new Lync Room System room account. Account can optionally
	be Enterprise Voice enabled.

.NOTES
  Version      	   	: 1.2 - See changelog at http://www.ehloworld.com/2462
	Wish list					: Define database, define OU path for user account
  Rights Required		: Organization Management or Recipient Management (Exchange)
  										CsAdministrator or CsUserAdministrator (Lync)
 										: If script is not signed, ExecutionPolicy of RemoteSigned (recommended) or
 										  Unrestricted (not recommended)
 										: If script is signed, ExecutionPolicy of AllSigned (recommended), RemoteSigned,
 										  or Unrestricted (not recommended)
  Sched Task Req'd	: No
  Lync Version			: 2013 through CU4
  Author(s)    			: © Pat Richard (pat@innervation.com) 	http://www.ehloworld.com @patrichard
  Dedicated Post		: http://www.ehloworld.com/2460
  Disclaimer   			: You running this script means you won't blame me if this breaks your stuff. This script is
  										provided AS IS without warranty of any kind. I disclaim all implied warranties including,
  										without limitation, any implied warranties of merchantability or of fitness for a particular
  										purpose. The entire risk arising out of the use or performance of the sample scripts and
  										documentation remains with you. In no event shall I be liable for any damages whatsoever
  										(including, without limitation, damages for loss of business profits, business interruption,
  										loss of business information, or other pecuniary loss) arising out of the use of or inability
  										to use the script or documentation.
  Acknowledgements 	: http://imaucblog.com/archive/2014/01/09/lync-room-system-lrs-account-creation-in-10-easy-steps/
  									: http://www.microsoft.com/en-us/download/details.aspx?id=39274
	Assumptions				: Exchange Server 2010 or 2013 exists in environment.
											Lync and AD modules are installed on the machine this script runs on.
  Limitations				: Office 365 does not support room resource accounts. So this script will not work with O365 based email
  Known issues			: None yet, but I'm sure you'll find some!

.LINK
	http://www.ehloworld.com/2460

.EXAMPLE
	.\New-CsLyncRoomSystem.ps1 -alias nycconfroom -name "New York City Conference Room" -upn "nycconfroom@contoso.com" -registrarpool "frontendpool.contoso.com"

	Description
	-----------
	Creates the room mailbox "nycconfroom", which will appear as "New York City Conference Room" in the GAL and Address Book. It will be assigned to the frontendpool.contoso.com front end pool.

.EXAMPLE
	.\New-CsLyncRoomSystem.ps1 -alias nycconfroom -name "New York City Conference Room" -upn "nycconfroom@contoso.com" -samaccountname nycconfroom -registrarpool "frontendpool.contoso.com"

	Description
	-----------
	Creates the room mailbox "nycconfroom", which will appear as "New York City Conference Room" in the GAL and Address Book. In this example, the SamAccountName is specified. It will be assigned to the frontendpool.contoso.com front end pool.

.EXAMPLE
	.\New-CsLyncRoomSystem.ps1 -alias nycconfroom -name "New York City Conference Room" -upn "nycconfroom@contoso.com" -samaccountname nycconfroom -registrarpool "frontendpool.contoso.com" -lineuri "tel:+19735551212;ext=1212"

	Description
	-----------
	Creates the room mailbox "nycconfroom", which will appear as "New York City Conference Room" in the GAL and Address Book. It will be assigned to the frontendpool.contoso.com front end pool. It will be Enterprise Voice enabled with a
	phone number of (973) 555-1212.

.INPUTS
	None. You cannot pipe objects to this script.
#>
#Requires -Version 2.0

[CmdletBinding(SupportsShouldProcess = $True)]
param (
	# Defines the Exchange alias that will be assigned to the room mailbox account.
	[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
	[ValidateNotNullOrEmpty()]
	[string] $Alias,

	# Defines the Name (display name) of the room mailbox. This is what appears in the Global Address List and Lync Address Book.
	[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
	[ValidateNotNullOrEmpty()]
	[string] $Name = "Executive Conference Room",

	# Defines the User Principal Name (UPN) of the room account.
	[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
	[ValidateNotNullOrEmpty()]
	[string] $UPN,

	# Defines the SamAccountName of the room account. This is optional, and only needed if the SamAccountName needs to be different than the alias.
	[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
	[string] $SamAccountName = "$Alias",

	# Defines the Lync Server front end pool. This is the pool where the account will be homed. It MUST be a Lync Server 2013 front end pool or Standard Edition server.
	[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
	[ValidateNotNullOrEmpty()]
	[ValidatePattern("(?=^.{1,254}$)(^(?:(?!\d+\.|-)[a-zA-Z0-9_\-]{1,63}(?<!-)\.){2,}(?:[a-zA-Z]{2,})$)")]
	[string] $RegistrarPool,

	# Defines the Lync LineURI of the room account. This should be in E.164 format. If not specified, the room account is not Enteprise Voice enabled.
	[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
	[string] $LineURI,

	# Defines the company name that will be configured for the account. This is important to avoid federated contacts from seeing "external network" instead of the company name.
	[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
	[ValidateNotNullOrEmpty()]
	[string] $CompanyName,

	# The AdditionalResponse parameter specifies the additional information to be included in responses to meeting requests. If not defined, the text from the Deployment Guide is used. This parameter supports HTML.
	[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
	[string] $ResponseText,

	# Defines the capacity of the conference room.
	[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
	[ValidatePattern("[0-9]")]
	[int] $ResourceCapacity,

	# Defines whether the subject is removed from the calendar entry. This can be critical if some users of the conference room should not see "sensitive" subjects for other meetings to be held in the same conference room.
	[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
	[switch] $DeleteSubject,

	# Specifies whether to include the reasons for accepting or declining a meeting in the response email message. When the EnableResponseDetails parameter is set to $true, the reasons for accepting or declining a meeting in the response email message are included.
	[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
	[bool] $EnableResponseDetails = $true
)

[string] $IncorrectLyncVersionWarning = @"
┌──────────────────────────────────────────────────────────┐
│         INVALID LYNC SERVER VERSION DETECTED             │
│         ====================================             │
│    Lync Room System must be homed on a Lync Server       │
│    2013 Front End Pool. It cannot be homed on an         │
│    earlier version. Script will now exit.                │
└──────────────────────────────────────────────────────────┘
"@

# The text below if the additional response included in the response email. It supports HTML.
$AdditionalResponse = @"
<p>If your meeting request was <b>declined</b>: Please disregard the rest of this message. It’s intended to help with using the technology in this conference room.</p>
<p>If your meeting request was <b>accepted</b>: Congratulations, you have scheduled a meeting with a Lync Room System meeting room!</p>
<p>Lync Room System (LRS) is a combination of software and hardware that enables rich meeting scenarios, including video conferencing, white boarding, PowerPoint sharing, and more. We are excited to have you try LRS, and we would love to hear your feedback!</p>
<h1>Key Scenarios:</h1>
<h2>Join Meeting</h2>
<p>If you're reading this mail, you've already scheduled a meeting. Just touch the Join button on your scheduled meeting to join it. Don’t see a join button? Make your meeting an online meeting in Outlook.</p>
<h2>Launch Whiteboard</h2>
<p>You can start whiteboarding, and then invite participants to share the whiteboard. You can also start whiteboarding from within a meeting.</p>
<h2>PowerPoint Sharing</h2>
<p>You can share your PPT slides with the room. To do this, upload the PPT file into the meeting from your machine (just as in Lync). From the room, you can then watch the PPT presentation, or take control and present.</p>
<h2>Display Modes</h2>
<p>Try using different display modes to see which one best fits your meeting.</p>
<p>If you run into any issues or have any questions, ideas, or feedback for the feature team, please contact your support team</p>
<p>Thanks!</p>
<h2>Frequently Asked Questions:</h2>
<p>Q: I see my meeting in the room’s calendar but there is no Join button.</p>
<p>A: Make sure that your meeting is an Online Meeting (Lync Meeting) in Outlook.</p>
"@

#region functions
function Get-ExchangeServersInSite { # http://www.mikepfeiffer.net/2010/04/find-exchange-servers-in-the-local-active-directory-site-using-powershell/ and http://www.mikepfeiffer.net/2010/04/import-ems-commands-from-an-exchange-2010-server-in-the-local-ad-site/
	[CmdletBinding(SupportsShouldProcess = $True)]
	param()
	$ADSite = [System.DirectoryServices.ActiveDirectory.ActiveDirectorySite]
	$siteDN = $ADSite::GetComputerSite().GetDirectoryEntry().distinguishedName
	$configNC = ([ADSI]"LDAP://RootDse").configurationNamingContext
	$search = New-Object DirectoryServices.DirectorySearcher([ADSI]"LDAP://$configNC")
	$objectClass = "objectClass=msExchExchangeServer"
	$version = "versionNumber>=1937801568"
	$site = "msExchServerSite=$siteDN"
	$search.Filter = "(&($objectClass)($version)($site))"
	$search.PageSize = 1000
	[void] $search.PropertiesToLoad.Add("name")
	[void] $search.PropertiesToLoad.Add("msexchcurrentserverroles")
	[void] $search.PropertiesToLoad.Add("networkaddress")
	$search.FindAll() | ForEach-Object {
		New-Object PSObject -Property @{
			Name = $_.Properties.name[0]
			FQDN = $_.Properties.networkaddress |
				ForEach-Object {if ($_ -match "ncacn_ip_tcp") {$_.split(":")[1]}}
			Roles = $_.Properties.msexchcurrentserverroles[0]
		}
	}
} # end function Get-ExchangeServersInSite

function Set-ModuleStatus {
	[CmdletBinding(SupportsShouldProcess = $True)]
	param	(
		[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true, Mandatory = $true, HelpMessage = "No module name specified!")]
		[ValidateNotNullOrEmpty()]
		[string] $name
	)
	PROCESS{
		# Executes once for each pipeline object
		# the $_ variable represents the current input object
		if (!(Get-Module -name "$name")) {
			if (Get-Module -ListAvailable | Where-Object Name -eq "$name") {
				Import-Module -Name "$name"
				# module was imported
				# return $true
			} else {
				# module was not available
				# return $false
			}
		} else {
			# Write-Output "$_ module already imported"
			# return $true
		}
	} # end PROCESS
} # end function Set-ModuleStatus

function Test-IsSigned {
<#
.SYNOPSIS
		Checks to see if the file containing this function is signed with a code signing certificate.

.DESCRIPTION
		Checks to see if the file containing this function is signed with a code signing certificate.

.NOTES
    Version							: 1.1 - See changelog at
		Wish list						: Better error trapping
    Rights Required			: Local administrator on server
    										: If script is not signed, ExecutionPolicy of RemoteSigned or Unrestricted
    										: If script is signed, ExecutionPolicy of AllSigned, RemoteSigned, or Unrestricted
    Sched Task Required	: No
    Lync Server Version	: N/A
    Author/Copyright		: © Pat Richard, Lync MVP - All Rights Reserved
    Email/Blog/Twitter	: pat@innervation.com 	http://www.ehloworld.com @patrichard
    Dedicated Blog Post	: http://www.ehloworld.com/1697
    Disclaimer   				: You running this script means you won't blame me if this breaks your stuff. This script is
    											provided AS IS without warranty of any kind. I disclaim all implied warranties including,
    											without limitation, any implied warranties of merchantability or of fitness for a particular
    											purpose. The entire risk arising out of the use or performance of the sample scripts and
    											documentation remains with you. In no event shall I be liable for any damages whatsoever
    											(including, without limitation, damages for loss of business profits, business interruption,
    											loss of business information, or other pecuniary loss) arising out of the use of or inability
    											to use the script or documentation.
    Assumptions					:
    Limitations					:
    Known issues				: None yet, but I'm sure you'll find some!
    Acknowledgements 		:

.LINK
		http://www.ehloworld.com/1697

.EXAMPLE
		PS C:\>

		Description
		-----------


.INPUTS
		None. You cannot pipe objects to this function.

.OUTPUTS
		Boolean output

#>
	[CmdletBinding(SupportsShouldProcess = $True)]
	param (
		[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
		[ValidateNotNullOrEmpty()]
		[string] $FilePath = $MyInvocation.ScriptName
	)

	BEGIN {
		# Executes once before first item in pipeline is processed
		# the $_ variable represents the current input object
	} # end BEGIN

	PROCESS {
		# Write-Output "Processing $_"
		# Executes once for each pipeline object
		# the $_ variable represents the current input object
		if ((Get-AuthenticodeSignature -FilePath $FilePath).Status -eq "Valid"){
			<#
			if ((Get-ExecutionPolicy) -ne "AllSigned"){
				Write-Warning "You could use an ExecutionPolicy of `"AllSigned`" when using this script. AllSigned is a higher level of security. For more information, see http://technet.microsoft.com/en-us/library/ee176961.aspx"
			}
			#>
			return $true
		}else{
			return $false
		}
	} # end PROCESS

	END {
		# Executes once after last pipeline object is processed
		# the $_ variable represents the current input object
	} # end END
} # end function Test-IsSigned

function Test-ScriptUpdates {
<#
.SYNOPSIS
		Checks the CreationTime parameter on the script itself. If it's over 30 days, it will prompt the user & optionally take them to the changelog for that script.

.DESCRIPTION
		Checks the CreationTime parameter on the script itself. If it's over 30 days, it will prompt the user & optionally take them to the changelog for that script.

.NOTES
    Version							: 1.0 - See changelog at
		Wish list						: Better error trapping
    Rights Required			: Local administrator on server
    										: If script is not signed, ExecutionPolicy of RemoteSigned (recommended)
    											or Unrestricted (not recommended)
    										: If script is signed, ExecutionPolicy of AllSigned (recommended, RemoteSigned,
    											or Unrestricted (not recommended)
    Sched Task Required	: No
    Lync Server Version	: N/A
    Exchange Version    : N/A
    Author/Copyright		: © Pat Richard, Lync MVP - All Rights Reserved
    Email/Blog/Twitter	: pat@innervation.com 	http://www.ehloworld.com @patrichard
    Dedicated Blog Post	:
    Disclaimer   				: You running this script means you won't blame me if this breaks your stuff. This script is
    											provided AS IS without warranty of any kind. I disclaim all implied warranties including,
    											without limitation, any implied warranties of merchantability or of fitness for a particular
    											purpose. The entire risk arising out of the use or performance of the sample scripts and
    											documentation remains with you. In no event shall I be liable for any damages whatsoever
    											(including, without limitation, damages for loss of business profits, business interruption,
    											loss of business information, or other pecuniary loss) arising out of the use of or inability
    											to use the script or documentation.
    Assumptions					:
    Limitations					:
    Known issues				: None yet, but I'm sure you'll find some!
    Acknowledgements 		:

.LINK


.EXAMPLE
		PS C:\> Get-ScriptUpdates -ChangeLogURL 1234 -Age 45

		Description
		-----------
		Executes the function, specifying the article number the user will be taken to if they choose yes to go online, as well as number of days that must have elapsed.

.INPUTS
		None. You cannot pipe objects to this function.

.OUTPUTS
		Text output

#>
	[CmdletBinding(SupportsShouldProcess = $True)]
	param (
		# Specifies the article number the user will be taken to if they choose yes to go online.
		[Parameter(Position = 0, ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $True)]
		[ValidateNotNullOrEmpty()]
		[int] $ChangelogUrl,

		# Specifies the number of days that must have elapsed before the prompt is displayed.
		[Parameter(Position = 1, ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $True)]
		[ValidateNotNullOrEmpty()]
		[int] $Age = 90,

		[Parameter(Position = 2, ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $True)]
		[ValidateNotNullOrEmpty()]
		[string] $FilePath = $MyInvocation.ScriptName
	)

	BEGIN{
		# Executes once before first item in pipeline is processed
		# the $_ variable represents the current input object
		[string]$Check4UpdatesPrompt = @"
┌──────────────────────────────────────────────────────────┐
│                Check For Updates?                        │
│            ==========================                    │
│    This script is more than $Age days old. Would you     │
│    like to check the script's website for a              │
│    newer version?                                        │
│                                                          │
└──────────────────────────────────────────────────────────┘
"@
	} # end BEGIN

	PROCESS{
		# Executes once for each pipeline object
		# the $_ variable represents the current input object
		# echo $_
		if (((Get-Date) - (Get-Item $filepath).CreationTime).TotalDays -gt $age){
			Write-Host $Check4UpdatesPrompt -ForegroundColor Green
			if ((Read-host "Go online? [y/n]") -imatch "y"){
				Start-Process "http://www.ehloworld.com/$ChangelogUrl"
			}
		}
	} # end PROCESS

	END{
		# Executes once after last pipeline object is processed
		# the $_ variable represents the current input object
	} # end END
} # end function Test-ScriptUpdates

#endregion functions

if (Test-IsSigned){
	Test-ScriptUpdates -ChangelogUrl 2460 -Age 90
}

$servers = New-Object System.Collections.ArrayList
Get-ExchangeServersInSite | ForEach-Object {[void]$servers.Add(($_.fqdn))}

#select a random server from the current site
if ($servers.count -gt 1) {
	$random = Get-Random -Minimum 0 -Maximum $servers.count
	$ExchangeServer = $servers[$random]
}	else {
	$ExchangeServer = $servers[0]
}

if ((Get-PSSession).ConfigurationName -ne "Microsoft.Exchange") {
	Write-Output "Establishing a session with Exchange"
	$Session = New-PSSession -ConfigurationName Microsoft.Exchange -ConnectionUri "http://$ExchangeServer/PowerShell/" -Authentication Kerberos
	# Import-PSSession $Session -AllowClobber -DisableNameChecking | Out-Null
	# Import-PSSession $Session -DisableNameChecking -CommandName New-Mailbox,Get-Mailbox,Set-Mailbox,Remove-Mailbox,Set-CalendarProcessing,Get-ADServerSettings -CommandType all | Out-Null
	Import-PSSession $Session -DisableNameChecking -CommandName *-Mailbox,Set-CalendarProcessing,Get-ADServerSettings -CommandType All | Out-Null
}

Write-Output "Selecting domain controller"
[string] $DomainController = (Get-ADServerSettings).DefaultConfigurationDomainController
Write-Output "Domain controller $DomainController selected"

Write-Output "Importing Lync module"
Set-ModuleStatus -Name Lync
Write-Output "Importing Active Directory module"
Set-ModuleStatus -Name ActiveDirectory

Write-Output "Checking Lync pool version"
if ((Get-CsService -PoolFqdn $RegistrarPool -Registrar).Version -ne 6){
	# Write-Output $RegistrarPool
	# Write-Output (Get-CsService -PoolFqdn $RegistrarPool -Registrar).Version
	Write-Output $IncorrectLyncVersionWarning
	exit
}

#region Exchange
if (!(Get-Mailbox -Identity $Alias -ea 0)){
	Write-Output "Creating mailbox $Alias ($name)"
	New-Mailbox -Name "$Name" -Alias $Alias -UserPrincipalName $UPN -SamAccountName $SamAccountName -Room -RoomMailboxPassword (Read-Host "Enter password for $alias" -AsSecureString) -EnableRoomMailboxAccount $true -DomainController $DomainController | Out-Null
	Write-Output "Setting description for user $Alias ($name)"
	if ($CompanyName){
		Set-ADUser -Identity $Alias -Description "Lync Room System equipped conference room" -Company "$CompanyName" -Server $DomainController
	}else{
		Set-ADUser -Identity $Alias -Description "Lync Room System equipped conference room" -Server $DomainController
	}
} elseif ((Get-Mailbox -Identity $Alias).ResourceType -ne "Room"){
	Write-Output "Mailbox $Alias ($name) already exists"
	Write-Output "Setting room parameters for $Alias ($name)"
	Set-Mailbox -Identity $Alias -type Room -RoomMailboxPassword (Read-Host "Enter password for $Alias" -AsSecureString) -EnableRoomMailboxAccount $true -DomainController $DomainController
} else {
	Write-Output "Mailbox $Alias already exists and is configured as a room"
}
if ($ResourceCapacity){
	Write-Output "Setting room capacity for $Alias ($name) to $ResourceCapacity"
	Set-Mailbox -Identity $Alias -ResourceCapacity $ResourceCapacity -DomainController $DomainController
}
if ($DeleteSubject){
	Write-Output "Setting Delete Subject for $Alias ($name)"
	Set-Mailbox -Identity $Alias -DeleteSubject $true -DomainController $DomainController
}
if ($ResponseText){
	$AdditionalResponse = $ResponseText
}

sleep -s 3
Write-Output "Configuring calendar processing for $Alias ($name)"

Set-CalendarProcessing -Identity $Alias -AutomateProcessing AutoAccept -AddOrganizerToSubject $false -RemovePrivateProperty $false -EnableResponseDetails $EnableResponseDetails –AddAdditionalResponse $true –AdditionalResponse $AdditionalResponse -DomainController $DomainController

Write-Output "Setting mailtip for $Alias ($name)"
Set-Mailbox -Identity $Alias -MailTip "This room is equipped with Lync Room System (LRS). Please make it a Lync Meeting to take advantage of the enhanced meeting experience from LRS." -DomainController $DomainController

#endregion Exchange

#region ActiveDirectory
sleep -s 3
Write-Output "Enabling AD account $Alias ($name)"
Enable-ADAccount $Alias -Server $DomainController
#endregion ActiveDirectory

#region Lync
Write-Output "Creating meeting room $Alias ($name)"
Enable-CsMeetingRoom -Identity $Alias -SipAddressType EmailAddress -RegistrarPool $RegistrarPool -DomainController $DomainController
Write-Output "Testing for meeting room $Alias ($name)"
if (Get-CsMeetingRoom $Alias -DomainController $DomainController -ErrorAction SilentlyContinue){
	Write-Output "$alias exists"
}else{
	Write-Output "$alias does NOT exist"
	break
}
if ($LineURI){
	if ($LineURI -notmatch "^tel:\+"){
		$LineURI = "tel:+"+$LineURI
	}
	Write-Output "Enterprise Voice enabling meeting room $Alias ($name)"
	Set-CsMeetingRoom -Identity $Alias -LineURI $LineURI -EnterpriseVoiceEnabled $true -DomainController $DomainController
}else{
	Write-Output "Checking Conferencing policy assigned to $Alias ($name)"
	$confpolicy = (Get-CsMeetingRoom $alias -DomainController $DomainController).ConferencingPolicy.FriendlyName
	if (! (Get-CsConferencingPolicy $confpolicy).AllowNonEnterpriseVoiceUsersToDialOut){
		Write-Warning "$Alias is not enterprise voice enabled, and is assigned to a conferencing policy that does not allow enterprise voice users to dial out. This is not recommended. Assign the meeting room to an appropriate policy to avoid issues."
	}
}

Write-Output "Checking user services policy for $Alias ($name)"
# start-sleep -s 60
[string] $uspolicy = (Get-CsMeetingRoom -Identity "$Alias" -DomainController $DomainController).UserServicesPolicy.FriendlyName
if (!$uspolicy){$uspolicy = "global"}
if ((Get-CsUserServicesPolicy $uspolicy).UcsAllowed){
	Write-Warning "$Alias is assigned to a user services policy that has UcsAllowed set to true. This is not recommended. Assign the meeting room to an appropriate policy to avoid issues."
}

# Set-User -Identity $Alias -Phone $LineURI -DomainController $DomainController

#endregion Lync

<#
if (Get-PSSession $session){
	Remove-PSSession $session
	Remove-Variable Session
}
Remove-Variable Servers
Remove-Variable DomainController
Remove-Variable ExchangeServer
Remove-Variable Random
#>

# SIG # Begin signature block
# MIIQBQYJKoZIhvcNAQcCoIIP9jCCD/ICAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUH7Bo3HwuhymT3AwOLUxNVQHM
# dgSggg1KMIIGnzCCBYegAwIBAgIQCSu3nvOdPAJrDHL+XciEEzANBgkqhkiG9w0B
# AQUFADBvMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMS4wLAYDVQQDEyVEaWdpQ2VydCBBc3N1cmVk
# IElEIENvZGUgU2lnbmluZyBDQS0xMB4XDTEzMDgyODAwMDAwMFoXDTE0MDkwNDEy
# MDAwMFowazELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAk1JMRkwFwYDVQQHExBTdGVy
# bGluZyBIZWlnaHRzMRkwFwYDVQQKExBJbm5lcnZhdGlvbiwgTExDMRkwFwYDVQQD
# ExBJbm5lcnZhdGlvbiwgTExDMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKC
# AQEA8dXC+TSUZsMisaialKJLat7AnmOF0HE+U49duaBJp/N4IXhjUZM9sXRkR/X0
# KAbNauNpNrs3TLKNpO7/RUinzrzsg8IhtBVXSrK7c0HDqJ7vG181it2qQO1yB9uD
# UV8frjprnNcR1w0qW9wFqclKlkyOxgiCANVoT6qXO9IxPZT28h+tMfBCsU7sZALs
# 3kPRMCQ1ZywngVSqEQ9rBYV3jW4EgvPEOmuhtnwgVB1Alr2V1dVAtbNxw1A7fQ8w
# J6EQSNAPVJQq7304QKKgGaTXtA2TwAjO5+m9ZltD/s0wXWYs0V1r8pOJOo1r17l4
# z91OWLPP+yfxvlnm5F1IXZxCbQIDAQABo4IDOTCCAzUwHwYDVR0jBBgwFoAUe2jO
# KarAF75JeuHlP9an90WPNTIwHQYDVR0OBBYEFB9N9j4QNDPcSscWMAVDKCOcXS/G
# MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzBzBgNVHR8EbDBq
# MDOgMaAvhi1odHRwOi8vY3JsMy5kaWdpY2VydC5jb20vYXNzdXJlZC1jcy0yMDEx
# YS5jcmwwM6AxoC+GLWh0dHA6Ly9jcmw0LmRpZ2ljZXJ0LmNvbS9hc3N1cmVkLWNz
# LTIwMTFhLmNybDCCAcQGA1UdIASCAbswggG3MIIBswYJYIZIAYb9bAMBMIIBpDA6
# BggrBgEFBQcCARYuaHR0cDovL3d3dy5kaWdpY2VydC5jb20vc3NsLWNwcy1yZXBv
# c2l0b3J5Lmh0bTCCAWQGCCsGAQUFBwICMIIBVh6CAVIAQQBuAHkAIAB1AHMAZQAg
# AG8AZgAgAHQAaABpAHMAIABDAGUAcgB0AGkAZgBpAGMAYQB0AGUAIABjAG8AbgBz
# AHQAaQB0AHUAdABlAHMAIABhAGMAYwBlAHAAdABhAG4AYwBlACAAbwBmACAAdABo
# AGUAIABEAGkAZwBpAEMAZQByAHQAIABDAFAALwBDAFAAUwAgAGEAbgBkACAAdABo
# AGUAIABSAGUAbAB5AGkAbgBnACAAUABhAHIAdAB5ACAAQQBnAHIAZQBlAG0AZQBu
# AHQAIAB3AGgAaQBjAGgAIABsAGkAbQBpAHQAIABsAGkAYQBiAGkAbABpAHQAeQAg
# AGEAbgBkACAAYQByAGUAIABpAG4AYwBvAHIAcABvAHIAYQB0AGUAZAAgAGgAZQBy
# AGUAaQBuACAAYgB5ACAAcgBlAGYAZQByAGUAbgBjAGUALjCBggYIKwYBBQUHAQEE
# djB0MCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wTAYIKwYB
# BQUHMAKGQGh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3Vy
# ZWRJRENvZGVTaWduaW5nQ0EtMS5jcnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0B
# AQUFAAOCAQEABTcbxhPxSJP7AKrF23wXbgelEGzjpewmm0BnHO9/V+9K4UE/3i5Z
# e3bX7OvUb/55zCHT4pvZl8/i8tuvFnrWZZcjGljT3W9SIVOuKNdlItXC8dvP67Hz
# OMZvSbdI5MU6sjd3vyYKznTMHG1Mz/x+BMh6rEnGsIaY4r8r4fkNfJTyTkww8AWh
# I16Pno3dWgKhfploPODtMxczJKA15VDFhQqM3oHHMojasXo5uGb5jlGiDFkZ3mDa
# /+gLUsJUjAzF27ZIy/jRgp0eMkqlgdxGa+gZUK73Ue6aTP2LW9SL/1fLYl8QkwRs
# MWG1faGdureT3Oh0EHH6UhREB/SxLJ+xLTCCBqMwggWLoAMCAQICEA+oSQYV1wCg
# viF2/cXsbb0wDQYJKoZIhvcNAQEFBQAwZTELMAkGA1UEBhMCVVMxFTATBgNVBAoT
# DERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEkMCIGA1UE
# AxMbRGlnaUNlcnQgQXNzdXJlZCBJRCBSb290IENBMB4XDTExMDIxMTEyMDAwMFoX
# DTI2MDIxMDEyMDAwMFowbzELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0
# IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEuMCwGA1UEAxMlRGlnaUNl
# cnQgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0EtMTCCASIwDQYJKoZIhvcNAQEB
# BQADggEPADCCAQoCggEBAJx8+aCPCsqJS1OaPOwZIn8My/dIRNA/Im6aT/rO38bT
# JJH/qFKT53L48UaGlMWrF/R4f8t6vpAmHHxTL+WD57tqBSjMoBcRSxgg87e98tzL
# uIZARR9P+TmY0zvrb2mkXAEusWbpprjcBt6ujWL+RCeCqQPD/uYmC5NJceU4bU7+
# gFxnd7XVb2ZklGu7iElo2NH0fiHB5sUeyeCWuAmV+UuerswxvWpaQqfEBUd9YCvZ
# oV29+1aT7xv8cvnfPjL93SosMkbaXmO80LjLTBA1/FBfrENEfP6ERFC0jCo9dAz0
# eotyS+BWtRO2Y+k/Tkkj5wYW8CWrAfgoQebH1GQ7XasCAwEAAaOCA0MwggM/MA4G
# A1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggrBgEFBQcDAzCCAcMGA1UdIASCAbow
# ggG2MIIBsgYIYIZIAYb9bAMwggGkMDoGCCsGAQUFBwIBFi5odHRwOi8vd3d3LmRp
# Z2ljZXJ0LmNvbS9zc2wtY3BzLXJlcG9zaXRvcnkuaHRtMIIBZAYIKwYBBQUHAgIw
# ggFWHoIBUgBBAG4AeQAgAHUAcwBlACAAbwBmACAAdABoAGkAcwAgAEMAZQByAHQA
# aQBmAGkAYwBhAHQAZQAgAGMAbwBuAHMAdABpAHQAdQB0AGUAcwAgAGEAYwBjAGUA
# cAB0AGEAbgBjAGUAIABvAGYAIAB0AGgAZQAgAEQAaQBnAGkAQwBlAHIAdAAgAEMA
# UAAvAEMAUABTACAAYQBuAGQAIAB0AGgAZQAgAFIAZQBsAHkAaQBuAGcAIABQAGEA
# cgB0AHkAIABBAGcAcgBlAGUAbQBlAG4AdAAgAHcAaABpAGMAaAAgAGwAaQBtAGkA
# dAAgAGwAaQBhAGIAaQBsAGkAdAB5ACAAYQBuAGQAIABhAHIAZQAgAGkAbgBjAG8A
# cgBwAG8AcgBhAHQAZQBkACAAaABlAHIAZQBpAG4AIABiAHkAIAByAGUAZgBlAHIA
# ZQBuAGMAZQAuMBIGA1UdEwEB/wQIMAYBAf8CAQAweQYIKwYBBQUHAQEEbTBrMCQG
# CCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wQwYIKwYBBQUHMAKG
# N2h0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJv
# b3RDQS5jcnQwgYEGA1UdHwR6MHgwOqA4oDaGNGh0dHA6Ly9jcmwzLmRpZ2ljZXJ0
# LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcmwwOqA4oDaGNGh0dHA6Ly9j
# cmw0LmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcmwwHQYD
# VR0OBBYEFHtozimqwBe+SXrh5T/Wp/dFjzUyMB8GA1UdIwQYMBaAFEXroq/0ksuC
# MS1Ri6enIZ3zbcgPMA0GCSqGSIb3DQEBBQUAA4IBAQB7ch1k/4jIOsG36eepxIe7
# 25SS15BZM/orh96oW4AlPxOPm4MbfEPE5ozfOT7DFeyw2jshJXskwXJduEeRgRNG
# +pw/alE43rQly/Cr38UoAVR5EEYk0TgPJqFhkE26vSjmP/HEqpv22jVTT8nyPdNs
# 3CPtqqBNZwnzOoA9PPs2TJDndqTd8jq/VjUvokxl6ODU2tHHyJFqLSNPNzsZlBjU
# 1ZwQPNWxHBn/j8hrm574rpyZlnjRzZxRFVtCJnJajQpKI5JA6IbeIsKTOtSbaKbf
# KX8GuTwOvZ/EhpyCR0JxMoYJmXIJeUudcWn1Qf9/OXdk8YSNvosesn1oo6WQsQz/
# MYICJTCCAiECAQEwgYMwbzELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0
# IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEuMCwGA1UEAxMlRGlnaUNl
# cnQgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0EtMQIQCSu3nvOdPAJrDHL+XciE
# EzAJBgUrDgMCGgUAoHgwGAYKKwYBBAGCNwIBDDEKMAigAoAAoQKAADAZBgkqhkiG
# 9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgorBgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIB
# FTAjBgkqhkiG9w0BCQQxFgQU0VzGmBh915KSsyR8+cnGANOTldgwDQYJKoZIhvcN
# AQEBBQAEggEAr+GfaByshm3SMBwvHZ2O8MbxBfCmKDzMW6EeM5yKmQCQDYYuyYG2
# 894c9bfhTEJ8H62dfKQf4k4SIDqj+tWlAvi4MNSfKAqguJAhxVEv8JWalfYIbYAd
# rNmSpb/I0ZRYgjVw5HLibS6i+7JW1VManiYA2ywGHNFO5xVjVVy+iY0MzJt08tB3
# ftbGsAnq7Ii8znjODjIHxW/Tsh65BP5USifSHVvdLMNo6xPJhqeGBq+SxZtX8yhT
# I8D/BCNTg6fVFm8MCDgY9dT+1mFFoppTB5nJ4vw+P3386dLSeP3gBRD2PuUBQ/kp
# 7GXpG1Ngt32WG1GDeQcUU7xazI13tQLlxw==
# SIG # End signature block
